{DelphiSim by Roeland van der Spek           }
{spekw@acm.org                               }
{(C) 1997                                    }
{v2.00                                       }
{Process Synchronization objects             }

unit synchronization;

interface

uses delphisim,classes;

Type TSimSignal = class(TSimWait)
     public
        function MustWait(S:TSimProcess):boolean; override;
        Procedure Signal;
     end;

     TSimSemaphore = class(TSimWait)
     protected
       s:Longint;
       a:Longint;
     public
       function MustWait(S:TSimProcess):boolean; override;
       procedure release(quantity:Longint);
     published
       property Available:Longint read a write a;
       property PouleSize:Longint read s write s;
     end;


procedure register;

implementation

{##############################################################################}
{########################## TSimSignal #### ###################################}
{##############################################################################}

Function TSimSignal.MustWait(S:TSimPRocess):Boolean;
begin
  AddProcess(s);
  result:=true;
end;

Procedure TSimSignal.Signal;
var i,j:Longint;
begin
  j:=ProcsPending;
  for i:=1 to j do ResumeProcess;
end;

{##############################################################################}
{########################## TSimSemaphore #####################################}
{##############################################################################}



Function TSimSemaphore.MustWait(S:TSimProcess):Boolean;
begin
  if a > 0 then
  begin
    dec(a);
    result:=false;
  end
  else
  begin
    AddProcess(s);
    result:=true;
  end;
end;

procedure TSimSemaphore.release(quantity:Longint);
var I:Longint;
begin
  if quantity+available > Poulesize then available :=Poulesize
  else available:=available+quantity;
  if quantity > ProcsPending then i:=ProcsPending
  else I:=quantity;
  while i>0 do
  begin
    ResumeProcess;
    dec(i);
    dec(a);
  end;
end;

Procedure register;
begin
  registercomponents('DelphiSim',[TSimSemaphore,TSimSignal]);
end;

end.
